@@ -1,14 +1,14 @@
 from __future__ import annotations
 
 import re
-from typing import TYPE_CHECKING, Any, Dict, List, Sequence
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Sequence
 
 import regex
 from sqlfluff.core import Lexer, Linter, Parser, SQLBaseError
 from sqlfluff.core.config import FluffConfig
 from sqlfluff.core.parser import CodeSegment, RegexLexer
 
-from easy_sql.sql_linter.rules import __all__
+from easy_sql.sql_linter.rules import all_rules
 from easy_sql.sql_linter.sql_linter_reportor import LintReporter
 from easy_sql.sql_processor.funcs import FuncRunner
 from easy_sql.sql_processor.report import SqlProcessorReporter
@@ -19,10 +19,10 @@ if TYPE_CHECKING:
 
 
 class SqlLinter:
-    def __init__(self, sql: str, include_rules: List[str] = None, exclude_rules: List[str] = None):
+    def __init__(self, sql: str, include_rules: Optional[List[str]] = None, exclude_rules: Optional[List[str]] = None):
         self.origin_sql = sql
         self.fixed_sql_list = []
-        self.supported_backend = ["spark", "postgres", "clickhouse", "bigquery"]
+        self.supported_backend = ["spark", "postgres", "clickhouse", "bigquery", "flink"]
         self.step_list = self._get_step_list()
         self.include_rules = include_rules
         self.exclude_rules = exclude_rules
@@ -56,7 +56,7 @@ class SqlLinter:
         step_list = step_factory.create_from_sql(self.origin_sql)
         return step_list
 
-    def _get_dialect_from_backend(self, backend: str = None) -> str:
+    def _get_dialect_from_backend(self, backend: Optional[str] = None) -> str:
         backend = backend or self._parse_backend(self.origin_sql)
         if backend == "spark":
             return "sparksql"
@@ -70,7 +70,7 @@ class SqlLinter:
         raise Exception("backend type so far is not supported for lint check")
 
     @staticmethod
-    def _update_included_rule_for_config(config: Dict[str, Any], dialect: str = None, rules=None):
+    def _update_included_rule_for_config(config: Dict[str, Any], dialect: Optional[str] = None, rules=None):
         if rules is None:
             rules = []
         if len(rules) > 0:
@@ -82,7 +82,7 @@ class SqlLinter:
                 config["core"]["rules"] = "core"
 
     @staticmethod
-    def _update_excluded_rule_for_config(config: Dict[str, Any], rules: List[str] = None):
+    def _update_excluded_rule_for_config(config: Dict[str, Any], rules: Optional[List[str]] = None):
         if rules is not None:
             config["core"]["exclude_rules"] = ",".join(rules)
         else:
@@ -110,7 +110,9 @@ class SqlLinter:
     def _lint_step_sql(
         self, step: Step, lexer: Lexer, parser: Parser, linter: Linter, log_error: bool
     ) -> List[SQLBaseError]:
+        assert step.target_config is not None
         self.fixed_sql_list.append(step.target_config.step_config_str)
+        vialations = []
         if step.select_sql:
             if step.is_template_statement():
                 self.fixed_sql_list.append(step.select_sql + "\n")
@@ -119,16 +121,19 @@ class SqlLinter:
                 sql = step.select_sql + "\n"
                 tokens, _ = lexer.lex(sql)
                 parsed = parser.parse(tokens)
+                assert parsed is not None
 
                 if self._check_lexable(tokens) and self._check_parsable(parsed):
                     result = linter.lint(parsed)
                     if log_error:
                         self.reporter.report_list_of_violations(result, step.target_config.line_no)
                     fixed_tree, violation = linter.fix(parsed)
+                    vialations += violation
                     self.fixed_sql_list.append(fixed_tree.raw)
                     return result
                 else:
                     self.fixed_sql_list.append(step.select_sql)
+        return vialations
 
     def _prepare_lexer(self, dialect: str):
         lexer = Lexer(dialect=dialect)
@@ -156,7 +161,9 @@ class SqlLinter:
             identifier_segement._template = regex.compile(template, regex.IGNORECASE)
         return parser
 
-    def _lint_for_easy_sql(self, backend: str, log_error: bool = True, config_path: str = None) -> List[SQLBaseError]:
+    def _lint_for_easy_sql(
+        self, backend: str, log_error: bool = True, config_path: Optional[str] = None
+    ) -> List[SQLBaseError]:
         lint_result = []
         dialect = self._get_dialect_from_backend(backend)
         lexer = self._prepare_lexer(dialect)
@@ -167,6 +174,7 @@ class SqlLinter:
         for step_no, step in enumerate(self.step_list):
             step_count = step_count + 1
             if log_error:
+                assert step.target_config is not None
                 self.reporter.report_message(
                     "=== Check step {} at line {} ===".format(step_no + 1, step.target_config.line_no)
                 )
@@ -175,7 +183,9 @@ class SqlLinter:
                 lint_result = lint_result + step_result
         return lint_result
 
-    def _lint_for_normal_sql(self, backend: str, log_error: bool = True, config_path: str = None) -> List[SQLBaseError]:
+    def _lint_for_normal_sql(
+        self, backend: str, log_error: bool = True, config_path: Optional[str] = None
+    ) -> List[SQLBaseError]:
         self.fixed_sql_list = []
         dialect = self._get_dialect_from_backend(backend)
         lexer = Lexer(dialect=dialect)
@@ -183,6 +193,7 @@ class SqlLinter:
         linter = self._prepare_linter(dialect, config_path=config_path)
         tokens, _ = lexer.lex(self.origin_sql)
         parsed = parser.parse(tokens)
+        assert parsed is not None
         lint_result = linter.lint(parsed)
         fixed_tree, violation = linter.fix(parsed)
         if log_error:
@@ -191,10 +202,11 @@ class SqlLinter:
         return lint_result
 
     def _parser_sql_header(self):
+        assert self.step_list[0].target_config is not None
         line_no = self.step_list[0].target_config.line_no
         return self.origin_sql.split("\n")[: line_no - 1]
 
-    def _prepare_linter(self, dialect: str, config_path: str = None):
+    def _prepare_linter(self, dialect: str, config_path: Optional[str] = None):
         if config_path:
             config = FluffConfig.from_path(config_path)
         else:
@@ -205,17 +217,19 @@ class SqlLinter:
             self._update_included_rule_for_config(default_config_dict, dialect=dialect, rules=self.include_rules)
             self._update_excluded_rule_for_config(default_config_dict, rules=self.exclude_rules)
             config = FluffConfig(configs=default_config_dict)
-        linter = Linter(config=config, user_rules=__all__)
+        linter = Linter(config=config, user_rules=all_rules)  # type: ignore
         return linter
 
     def lint(
-        self, backend: str, log_error: bool = True, easysql: bool = True, config_path: str = None
+        self, backend: str, log_error: bool = True, easysql: bool = True, config_path: Optional[str] = None
     ) -> List[SQLBaseError]:
         if easysql:
             return self._lint_for_easy_sql(backend, log_error, config_path=config_path)
         else:
             return self._lint_for_normal_sql(backend, log_error, config_path=config_path)
 
-    def fix(self, backend: str, log_linter_error: bool = False, easy_sql: bool = True, config_path: str = None):
+    def fix(
+        self, backend: str, log_linter_error: bool = False, easy_sql: bool = True, config_path: Optional[str] = None
+    ):
         self.lint(backend, log_linter_error, easy_sql, config_path=config_path)
         return "\n".join(self.fixed_sql_list)
